;******************************************************************************
;(C) Copyright 2021 SuperDARN Canada, University of Saskatchewan
;
; Author: Pasha Ponomarenko (University of Saskatchewan)
;
; This program is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; For the GNU General Public License
; see <http://www.gnu.org/licenses/>.
;
; Modifications:
;
;******************************************************************************

;---------------------------------------------------------------------------------------------------------------------------------
;This is a code to perform statistical comparison of FITACF data processed
;with different version of the software (e.g., FITACF2.5 and FITACF3.0)
;This is achived by calcualting 2D histograms for a following set of fitted parameters:
;(1) line-of-sight velocity (fit.v)
;(2) lag 0 power (fit.pwr0)
;(3) fitted power (Lorenzian, fit.p_l)
;(4) fitted spectral width (Lorentzian, fit.w_l)
;(5) elevaiton (fit.elv)
;If necessary, this list can be asily mdified.
;
;The original FITACF data processed by FITACF2 and FITACF3 should be stored in two separate folders which are set up through DATA_DIR2 and DATA_DIR3 variables, respectively.
;
;The obtained histograms and related metadata are stored as 2D arrays in OUT structure which is sved in a *.SAV file and placed in the output folder set up through a variable OUT_DIR.
;
;Ionopsheric and ground scatter histograms are stored separately and can be distinguished in a following way:
;OUT[0] -- ionospheric scatter
;OUT[1] -- ground scatter
;
;Currently there are following 2D histograms generated:
;H_VV 			-- line-of-sight velocity (FIT.V)
;H_PP			-- fitted lag 0 "power" (FIT.P_L)
;H_WW			-- Lorentzian spectral width (FIT.W_L)
;H_P0P0			-- measured lag 0 power (FIT.PWR0)
;H_EE			-- elevating angle (FIT.ELV)
;
;	For example, the velocity hisotgram for ionospheric scatter is accessed as OUT[0].H_VV
;
;X_V, X_P, X_E, X_W -- 2D histogram axes values for velocity [m/s], lag 0"power" [dB], elevatiion angle [deg] and spectral width [m/s], respectively




;----------------------
;Required environnment:
;----------------------
;The program requires RST with IDL functionality enabled.
;
;-----------------------
;Supplementary programs:
;-----------------------
;READ_FITACF, FIT_CLIPPED_BLANK 			-- reading fitted data into standard structures
;HISTO_2D                       			-- basic code for calculating 2D histogramms
;GENERAL_STATISTICS2D 						-- generating 2D histograms of selceted paramters from a single FITACF file.
;STAT_ARR_GENERATE2D 						-- creating a blank output data structure



;------------------------------------------------------------------

pro fitacf2_3_comparison_2d;, st_id=st_id

      ;List of radar IDs to process
      id_list=['rkn'];, 'inv', 'cly', 'pgr']

      ;First year to analyse
      year_st=2017
      ;Last year to analyse
      year_fn=2017
      ;First month to analyse:
      month_st=2;
      ;Last month to analyse:
      month_fn=2;

      ;Beam selection
      ;Note: to process all beams set this parameter to -1
      beam=-1
      

      ;Input data folders with FITACF2 and FITCAF3 files
      data_dir2='/data/temp/data/fit/fitacf2/new/'
      data_dir3='/data/temp/data/fit/fitacf3/new/'

      ;Output dta folder
      out_dir='/data/temp/data/compar23_output/'



;---------------------------------------------------
;Cycle for radars
;---------------------------------------------------
for id_inx=0,n_elements(id_list)-1 do begin
      st_id=id_list[id_inx]

      
      ;Setting lower/upper limits and bin sizes for different parameters:

      v_min=-2000.
      v_max=2000.
      dv=10.

      ;Calculating number of bins and generating bin label arrays for plotting
      n_v=round(v_max-v_min)/dv
      x_v=v_min+dv*findgen(n_v)



      w_min=0.
      w_max=600.
      dw=5.
      n_w=round(w_max-w_min)/dw
      x_w=w_min+dw*findgen(n_w)

      w_e_min=0.
      w_e_max=1000.
      d_w_e=10.
      n_w_e=round(w_e_max-w_e_min)/d_w_e
      x_w_e=w_e_min+d_w_e*findgen(n_w_e)



      p_min=-5.
      p_max=60.
      dp=0.5
      n_p=round(p_max-p_min)/dp
      x_p=p_min+dp*findgen(n_p)


      e_min=0.
      e_max=50.
      de=1.
      n_e=round(e_max-e_min)/de
      x_e=e_min+de*findgen(n_e)




      stat_arr_generate2d,$
        v_min=v_min, $
        v_max=v_max, $
        dv=dv, $
        w_min=w_min, $
        w_max=w_max, $
        dw=dw, $
        p_min=p_min, $
        p_max=p_max, $
        dp=dp, $
        e_min=e_min, $
        e_max=e_max, $
        de=de, $
        out_stat=out_stat_single

        out=replicate(out_stat_single,2)


      ;---------------------------------------------------
      ;Cycle for years
      ;---------------------------------------------------
      for year=year_st, year_fn do begin
          ;---------------------------------------------------
          ;Cycle for months
          ;---------------------------------------------------
          for mnth=month_st,month_fn do begin
            mask=string(year,'(i4.4)')+string(mnth,'(i2.2)')+'*'+st_id+'.fitacf'
      	if st_id eq 'mcm' or st_id eq 'sps' or st_id eq 'ade' or st_id eq 'adw' then mask=string(year,'(i4.4)')+string(mnth,'(i2.2)')+'*'+st_id+'.a.fitacf'
      	if st_id eq 'kod' or st_id eq 'ksr' then mask=string(year,'(i4.4)')+string(mnth,'(i2.2)')+'*'+st_id+'.*.fitacf'
            ;Looking for files in FITACF2 folder
            files2=file_search(data_dir2, mask, count=count2)

            if (count2 ge 1) then begin
              ;---------------------------------------------------
              ;Cycle for files (one month)
              ;---------------------------------------------------
              for i=0, count2-1 do begin
                ;Looking for files with the same name in FITACF3 folder
                mask3=strmid(files2[i], strlen(data_dir2))
                file3_cur=file_search(data_dir3,mask3, count=count_3)
                if count_3 ne 1 then begin
                  print, 'There is no '+file3_cur+' file in FITACF3 folder!'
                  continue
                endif

                t=systime(1)
                print, mask3
                read_fitacf, fname=files2[i], prm=prm2, fit=fit2, beam=beam, max_range=70
                read_fitacf, fname=file3_cur, prm=prm3, fit=fit3, beam=beam, max_range=70
                ;CP_ID selection
                inx_cp=where(((abs(prm2.cp) ge 3501 and abs(prm2.cp) le 3505) or abs(prm2.cp) eq 151 $
      			or abs(prm2.cp) eq 153 or abs(prm2.cp) eq 503 or abs(prm2.cp) eq 157 or abs(prm2.cp) eq 150))

                if inx_cp[0] gt -1 then begin
                  prm2=prm2[inx_cp]
                  fit2=fit2[inx_cp]
                  prm3=prm3[inx_cp]
                  fit3=fit3[inx_cp]

                  lags=(prm2[0].lag[*,1]-prm2[0].lag[*,0])
                  tau= lags[1:prm2[0].mplgs-1]*prm2[0].mpinc/1e6
                  ;sum=sqrt(1./total(tau^2))/4.
                  ;---------------------------------------------------
                  ;Cycle for ionospheric and ground scatter
                  ;---------------------------------------------------
                  for gflg=0,1 do begin
                    general_statistics2d, prm2=prm2,prm3=prm3, fit2=fit2,fit3=fit3, grnd=gflg,$
                      v_min=v_min, $;End of the cycle for stations
                      v_max=v_max, $
                      dv=dv, $
                      w_min=w_min, $
                      w_max=w_max, $
                      dw=dw, $
                      e_min=e_min, $
                      e_max=e_max, $
                      de=de, $
                      p_min=p_min, $
                      p_max=p_max, $
                      dp=dp, $
                      out_stat=out_stat


                        out[gflg].h_vv=out[gflg].h_vv+out_stat.h_vv
                         out[gflg].h_pp=out[gflg].h_pp+out_stat.h_pp
                          out[gflg].h_p0p0=out[gflg].h_p0p0+out_stat.h_p0p0
                           out[gflg].h_ww=out[gflg].h_ww+out_stat.h_ww
                            out[gflg].h_ee=out[gflg].h_ee+out_stat.h_ee
                             out[gflg].n_tot=out[gflg].n_tot+out_stat.n_tot
                  ;---------------------------------------------------
                  ;End of the cycle for ionospheric and ground scatter
                  ;---------------------------------------------------
                  endfor
                endif
              ;---------------------------------------------------
              ;End of the cylce for files
              ;---------------------------------------------------
              endfor

            endif
        ;---------------------------------------------------
        ;End of the cycle for months
        ;---------------------------------------------------
        endfor

        if  (total(out.h_vv)) ne 0 then $
        save, out, x_v, x_p, x_e, x_w, month_st, month_fn, year_st,year_fn, beam,  st_id, $
      filename=out_dir+string(year_st,'(i4.4)')+string(month_st,'(i2.2)')+st_id+'_save_stat_2d.sav' $
      else print, "No data for "+st_id

      ;End of the cycle for years
      endfor

;End of the cycle for radars
endfor


end




