
;******************************************************************************
;(C) Copyright 2021 SuperDARN Canada, University of Saskatchewan
;
; Author: Pasha Ponomarenko (University of Saskatchewan)
;
; This program is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; For the GNU General Public License
; see <http://www.gnu.org/licenses/>.
;
; Modifications:
;
;******************************************************************************



;---------------------------------------------------------------------------------------------------------------
;READ_FITACF
;
;Written by Pasha PONOMARENKO 24 August 2021.
;
;This is a reading routine to extract fitted data
;from both old (.fit) and new (.fitacf) types of files.
;It automatically detects the data type from the file extension.
;This routine can read data for all beams or for a single beam.
;----------------------------------------------------------------------------------------------------------------


pro read_fitacf, fname=fname, prm=prm, fit=fit, beam=beam, max_range=max_range

;INPUT PARAMETERS:
;FNAME        - filename including the full path to the data folder (optional)
;               If this keyword is not set, the routine opens a file seleciton widget in the designated directory
;MAX_RANGE    - maximum range gate (optional)
;               If this keyword is not set, then the data will be read from all filled range gates (i.e., MAX_RANGE=PRM.NRANG)
;BEAM         - beam number (negative -- all beams)
;               ATTENTION: If this keyword is not set or set to 0, only beam 0 data will be read.



;OUTPUT PARAMETERS:
;PRM         - output parameter structure
;FIT         - output fitted data structure

;Setting a default data folder
if ~keyword_set(dir_fit) then dir_fit='/home/data'

;Opening a file selection widget if the filname is not provided:
if ~keyword_set(fname) then fname=dialog_pickfile(filter='*.fit*', get_path=g_path, path=dir_fit, /read)

if ~keyword_set(beam) then beam=0;

;Detecting the file type from the filename extension by reading the last three characters: old -- 'fit', new -- 'acf'
type=STRMID(fname, 2 , 3, /REVERSE_OFFSET)

;An output array with n_rec=30000 blanck records to be filled is created.
;This array size should be enough for a 24-hr high-resolution mode file,
;but if it is insufficient, the arrays will be extended (see below).
;Creating a large array and filling it in is much faster than extending the output array record-by-record.

n_rec_ini=long(30000)

n_rec=n_rec_ini


;Creating fit and prm structures for old type data
if (type eq 'acf' or type eq 'ACF') then begin
fitfp=FitOpen(fname,/read);
    ;Determining the nuber of gates in the data file
    ;by reading the first record and looking into the PRM structure:
		dumb=fitRead(fitfp, prmcur, fitcur)
		;Setting maximum range gate to the default if it has not been set beforehand
		if ~keyword_set(max_range) then max_range=prmcur[0].nrang
		;Creating multi-record arrays of FIT and PRM structures using FIT_CLIPPED routine for
		;restricting the number of variables to those which are actually needed:
		fit_clipped_blank, fit=fit1, max_range=max_range
		fit=replicate(fit1, n_rec)
		prm=replicate(prmcur, n_rec)
close, fitfp
endif

if (type eq 'fit' or type eq 'fit') then begin
fitfp=oldFitOpen(fname);
		dumb=oldfitRead(fitfp, prmcur, fitcur)
		if ~keyword_set(max_range) then max_range=prmcur[0].nrang
		fit_clipped_blank, fit=fit1, max_range=max_range, parms=simple_stat
		fit=replicate(fit1, n_rec)
		prm=replicate(prmcur, n_rec)
s=oldfitClose(fitfp)
endif

;Record counter
count=long(0)


if (type eq 'acf' or type eq 'ACF') then begin
fitfp=fitOpen(fname,/read)

;Reading and filling the array until there is no more records
while fitRead(fitfp,prmcur,fitcur) ne -1 do begin
	if beam lt 0  or prmcur.bmnum eq beam then begin
			struct_assign, fitcur, fit1
			;Doubling the output arrays in size if their initial dimensions
			;are smaller than the nuber of records in the file
			if count eq n_rec then begin
			   fit=[fit,replicate(fit1, n_rec)]
         prm=[prm,replicate(prmcur, n_rec)]
			   n_rec=n_rec+n_rec_ini
			endif
			fit(count)=fit1
			prm(count)=prmcur
			count=count+1
	endif
endwhile
close, fitfp
endif

if (type eq 'fit' or type eq 'FIT') then begin
fitfp=oldfitOpen(fname)
while oldfitRead(fitfp,prmcur,fitcur) ne -1 do begin
	if beam lt 0  or prmcur.bmnum eq beam then begin
			struct_assign, fitcur, fit1
			;Doubling the output arrays in size if their initial dimensions
			;are smaller than the nuber of records in the file
			if count eq n_rec then begin
			   fit=[fit,replicate(fit1, n_rec)]
			   prm=[prm,replicate(prmcur, n_rec)]
			   n_rec=n_rec+n_rec_ini
			endif
			fit(count)=fit1
			prm(count)=prmcur
			count=count+1
		endif
endwhile


s=oldFitClose(fitfp)
endif

;Removing the empty records fields
if count gt 0 then begin
	fit=fit(0:count-1)
	prm=prm(0:count-1)
endif else begin
	fit=fit[0]
	prm=prm[0]
endelse



close, /all


end
